/**********************************************************************
 * These are functions that allow you to step through the database
 * names on a Notes server. On your first call to GetNextDbName,
 * a list of databases is created and stored, and subsequent calls
 * to the function using the same server and directory name will
 * grab the next database name, until there are no more databases
 * to get. Subdirectories are recursively searched.
 *
 * If you want to get the local databases, use "" as the serverName
 * parameter in the GetNextDbName function. Also, if you want to
 * start your search from the Notes data directory, use "" as the
 * dirName parameter (otherwise the dirName should be relative to the
 * Notes data directory).
 *
 * Written by Julian Robichaux
 * http://www.nsftools.com
 *
 * You may use this code in any way you'd like, as long as you don't
 * hold me liable for anything, and you don't tell people that you 
 * wrote it yourself. If you're feeling really generous, please include
 * my name and website whenever you use this code in your programs.
 * Thanks!
 **********************************************************************/


#ifdef __cplusplus
extern "C" {
#endif

#define MAXLIST 1024
#define MAXSTR 256

/* structure for holding the information */
typedef struct _FILE_LIST {
	char	serverName[MAXSTR];
	char	fileName[MAXSTR];
	char	fullPath[MAXSTR];
	char	physicalPath[MAXSTR];
} FILE_LIST, *PFILE_LIST;

/* Function prototypes */
/* This is the main function you'll use. You can call it in a loop like this:
 *	while (GetNextDbName(server_name, "", dbName, &nError))
 *	{
 *		GetDbNameOnly(dbNameOnly);
 *		if ((IsWildcardMatch(dbFilter, dbName)) || (IsWildcardMatch(dbFilter, dbNameOnly)))
 *			DoSomething(server_name, dbName);
 *	}
 */
BOOL GetNextDbName (char *serverName, char *dirName, char *retDbName, STATUS *retError);


/* This gets just the database name (without the path) for the current database
 * in the list.
 */
void GetDbNameOnly (char *retDbName);


/* This function gets the full disk path (like c:\notes\data\dbname.nsf) for the
 * current database in the list. May only work for Notes R5 or later.
 */
void GetDbDiskPath (char *retDbPath);

#ifdef __cplusplus
}
#endif

