/********************************

PROGRAM: aclhelp

ACLHelp 1: Ian Cherrill
Created: November 2000

ACLHelp 2: Ian Cherrill, Nicolas Laureaux
Created: September 2002

DESCRIPTION: aclhelp lets the administrator
put a name into an ACL from the console - 
useful if you've removed yourself from 
the ACL by mistake.

In ACLHelp 2
------------
Nicolas Laureaux
added the ability to apply the ACL name to every 
database in a directory

N.Laureaux : prise en compte d'un rpertoire au lieu
d'un nom de fichier

Ian added the ability to set access levels
for readers, authors and editors etc
and to remove entries
and list the ACL and use *.ntf *.nsf

Nicolas modified the test of the database/directory
to correctly interpret .ntf and.box as databases

*********************************/

#include "aclhelp.h"
#include "getdblist.h"
#include "wildcard.h"

/* Function prototypes */
STATUS LNPUBLIC file_action (void *, SEARCH_MATCH *, ITEM_TABLE *);
STATUS processDirectory( char *, t_acl_info *);
STATUS updateACL(char *, t_acl_info *);
STATUS LNPUBLIC get_file_summary (ITEM_TABLE *, char *);
STATUS ListEntries (HANDLE, t_acl_info *);
void GetAccessLevelStr (WORD, char *);
void LNPUBLIC ListCallback (void *, char *, WORD, ACL_PRIVILEGES *, WORD);
void LogMsg (char *);
void LogError (STATUS);

/******** Main routine ******/
#ifdef _DEBUG
LAPI_MAIN
{

#else

STATUS LNPUBLIC AddInMain (HMODULE hModule, int argc, char *argv[])
{
	HANDLE hOldStatusLine; /* handle to initial default status line*/
	HANDLE hStatusLineDesc; /* handle to new default status line */
	HMODULE hMod; /* add-in task's module handle */

#endif
	WORD acl_level = ACL_LEVEL_MANAGER;
	int pos = 3;
	char msg[MSG_LENGTH];
	/** variable definitions for debug or release code **/
	STATUS error=NOERROR; /* return code from API calls */
	char entryname[MAXUSERNAME], canonname[MAXUSERNAME], pathname[MAXPATH];
	t_acl_info acl_info;
	int i;
	WORD retLen;

	/** code to initialise program **/
#ifdef _DEBUG 
	LAPI_INIT(error);
	if (error) LAPI_INIT_ERROR;
	LogMsg("Starting AClHelp...");
#else
	AddInQueryDefaults (&hMod, &hOldStatusLine);

	AddInDeleteStatusLine (hOldStatusLine);

	hStatusLineDesc = AddInCreateStatusLine(PROGRAM_NAME);

	AddInSetDefaults (hMod, hStatusLineDesc);

	AddInSetStatusText("Initialising");

	/* Initialization is complete */

	AddInSetStatusText("Idle"); /* 'Idle' replaces 'Initialising' */
#endif

	/*the parameters are recorded in the argv[] array
	and the number of parameters (including the program name)
	are given by the counter argc
	this is the same in just about any C program

	our parameters after "LOAD" the command must be
	#0 aclhelp
	#1 database name
	#2 could be a switch to determine access level
	#2 or an acl entry name - a user or a group
	greater than #2=other parts of the entry name string
	*/

	if (argc > 1 && strcmp(argv[1], "-help") == 0)
	{
		LogMsg ("parameter #1: directory, filename, *.nsf or *.ntf");
		LogMsg ("parameter #2: optional switches eg. -manager -person");
		LogMsg ("(no switches defaults to unspecified manager)");
		LogMsg ("parameter #3: user/group name to add or ? to list entries");
		LogMsg ("example: load aclhelp log.nsf Ian Cherrill/Acme");
		LogMsg ("adds user as manager to log.nsf");
		LogMsg ("example: load aclhelp mail* Nicolas Laureaux/Acme");
		LogMsg ("adds user as manager to databases in mail directory");
		LogMsg ("example: load aclhelp intranet\\homepage.nsf -reader -persongroup All Users");
		LogMsg ("adds the All Users person group as reader of the database");
		LogMsg ("possible switches:");
		LogMsg ("-manager -designer -editor -author -reader -depositor");
		LogMsg ("-noaccess -remove -nocreate -nodelete");
		LogMsg ("-person -server -mixedgroup -persongroup -servergroup");
		LogMsg ("-serveradmin -serveradmin+");
		return (NOERROR);

	}

	if (argc < 3) /*first three are required ie. aclhelp dbaname entryname*/
	{
		LogMsg ("parameters: <db or directory> <switches> <user or group name>");
		LogMsg ("for more information use -help");
		return NOERROR;
	}

	strcpy (pathname, argv[1]);

	acl_info.flags = 0;
	acl_info.remove_entry = FALSE;

	/*if user has entered a switch, use this to determine access level required*/
	/*the program defaults to manager access*/
	for (i = 2; i < argc; i++)
	{
		if (strcmp(argv[i], "-remove") == 0)
		{
			acl_info.remove_entry = TRUE;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-noaccess") == 0)
		{
			acl_level = ACL_LEVEL_NOACCESS; /*remove from ACL*/
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-reader") == 0)
		{
			acl_level = ACL_LEVEL_READER;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-author") == 0)
		{
			acl_level = ACL_LEVEL_AUTHOR;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-editor") == 0)
		{
			acl_level = ACL_LEVEL_EDITOR;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-designer") == 0)
		{
			acl_level = ACL_LEVEL_DESIGNER;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-depositor") == 0)
		{
			acl_level = ACL_LEVEL_DEPOSITOR;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-manager") == 0)
		{
			acl_level = ACL_LEVEL_MANAGER;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-nocreate") == 0)
		{
			acl_info.flags |= ACL_FLAG_AUTHOR_NOCREATE;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-nodelete") == 0)
		{
			acl_info.flags |= ACL_FLAG_NODELETE;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-person") == 0)
		{
			acl_info.flags |= ACL_FLAG_PERSON;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-persongroup") == 0)
		{
			acl_info.flags |= ACL_FLAG_GROUP;
			acl_info.flags |= ACL_FLAG_PERSON;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-server") == 0)
		{
			acl_info.flags |= ACL_FLAG_SERVER;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-serveradmin") == 0 || strcmp(argv[i], "-serveradmin+") == 0)
		{
			acl_info.flags |= ACL_FLAG_ADMIN_SERVER;
			acl_info.flags |= ACL_FLAG_SERVER;
			if (strcmp(argv[i], "-serveradmin+") == 0)
				acl_info.adminsrv_controls_fields = TRUE;
			else acl_info.adminsrv_controls_fields = FALSE;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-servergroup") == 0)
		{
			acl_info.flags |= ACL_FLAG_GROUP;
			acl_info.flags |= ACL_FLAG_SERVER;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-mixedgroup") == 0)
		{
			acl_info.flags |= ACL_FLAG_GROUP;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-scriptagents") == 0 || strcmp(argv[i], "-javaagents") == 0)
		{
			acl_info.flags |= ACL_FLAG_CREATE_LOTUSSCRIPT;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-privateagents") == 0)
		{
			acl_info.flags |= ACL_FLAG_CREATE_PRAGENT;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-privateviews") == 0 || strcmp(argv[i], "-privatefolders") == 0)
		{
			acl_info.flags |= ACL_FLAG_CREATE_PRFOLDER;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-sharedviews") == 0 || strcmp(argv[i], "-sharedfolders") == 0)
		{
			acl_info.flags |= ACL_FLAG_CREATE_FOLDER;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-publicreader") == 0)
		{
			acl_info.flags |= ACL_FLAG_PUBLICREADER;
			pos++; /*start the entry name at the next part of the command line*/
		}
		if (strcmp(argv[i], "-publicwriter") == 0)
		{
			acl_info.flags |= ACL_FLAG_PUBLICWRITER;
			pos++; /*start the entry name at the next part of the command line*/
		}
	}

/* here are the possible acl flags
ACL_FLAG_AUTHOR_NOCREATE
ACL_FLAG_NODELETE
ACL_FLAG_CREATE_PRAGENT
ACL_FLAG_CREATE_PRFOLDER
ACL_FLAG_CREATE_FOLDER
ACL_FLAG_CREATE_LOTUSSCRIPT
ACL_FLAG_PUBLICREADER
ACL_FLAG_PUBLICWRITER
ACL_FLAG_PERSON
ACL_FLAG_GROUP
ACL_FLAG_SERVER
*/
	strcpy (entryname, argv[pos-1]);
	
	/*if user has used spaces eg. Ian Cherrill, get other parts of the entry name*/
	for (i=pos /*start with fourth part of command string*/; i<argc; i++)
	{
		strcat (entryname, " ");
		strcat (entryname, argv[i]);
	}

	if (strlen (entryname) < 1)
	{
		LogMsg("please specify an entry name");
		return NOERROR;
	}

	if (entryname[0] == '-' && strcmp(entryname, ENTRYTYPE_DEFAULT) != 0)
	{
		sprintf(msg, "%s could not be understood", entryname);
		LogMsg( msg );
		return NOERROR;
	}

	sprintf(msg, "processing ACL in %s for %s",pathname,entryname);
	LogMsg( msg );
	
	/*convert entryname to canonical eg. CN=Ian Cherrill/O=4NF */
	/* if you added Ian Cherrill/4NF to an ACL it would be corrupt*/
	if (error = DNCanonicalize (0L, NULL, entryname, canonname, MAXUSERNAME, &retLen))
	{
		LogError (error);
		return (NOERROR);
	}

	acl_info.level = acl_level;
	strcpy (acl_info.canonname, canonname);

	if (error = processDirectory (pathname, &acl_info))
	{
		LogError (error);
		return (NOERROR);
	}

	LogMsg("finished");


#ifndef _DEBUG
	AddInSetStatusText("Terminating");
#endif

	/* End of main routine. */
	return (NOERROR);
}

/************************************************************/

STATUS processDirectory( char *database_name_requested, t_acl_info *acl_info)
{
	STATUS error = NOERROR;
	char        database_name_found[MAXPATH];
	BOOL is_a_match;
	
	while (GetNextDbName ("", "", database_name_found, &error))
	{
		is_a_match = IsWildcardMatch(database_name_requested, database_name_found, FALSE);

		if (is_a_match)
		{
			if (error = updateACL (database_name_found, acl_info ))
			{
				LogError (error);
				return (NOERROR);
			}
		}
	}	
	return(NOERROR);
}

/************************************************************/
STATUS updateACL(char *pathname, t_acl_info *acl_info)
{
	HANDLE hACL;
	ACL_PRIVILEGES PrivBits = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0}; 
	DWORD dwACLFlags;
	STATUS error=NOERROR; /* return code from API calls */
	DBHANDLE hDb=NULL;
	char full_netpath[MAXPATH];
	char msg[MSG_LENGTH];
	int i;
	int dot_pos = strlen(pathname)-4;

	if (error = OSPathNetConstruct(NULL, "", pathname,
					full_netpath))
	{
		LogError (error);
		return (NOERROR);
	}

	strcpy (msg, full_netpath);

	/*open the db in question*/
	if (error = NSFDbOpen (full_netpath, &hDb))
	{
		strcat (msg, " database could not be opened"); 
		LogMsg(msg);
		LogError (error);
		return (NOERROR);
	}

	/* read the existing ACL from the open database */
	if (error = NSFDbReadACL(hDb, &hACL))
	{
		NSFDbClose (hDb); /*close the open database*/
		strcat (msg, " ACL could not be read"); 
		LogMsg(msg);
		LogError (error);
		return (NOERROR);
	}

	/*is just a list required?*/
	if (strcmp(acl_info->canonname, ENTRYTYPE_LIST) == 0)
	{
		char level_name[SHORT_STRING];

		GetAccessLevelStr(acl_info->level, level_name);

		strcpy (msg, "listing ACL entries for ");
		strcat (msg, pathname);
		strcat (msg, " at level: ");
		strcat (msg, level_name);
		LogMsg(msg);

		if (error = ListEntries(hACL, acl_info)) LogError (error);

		LogMsg ("finished listing ACL");
		return NOERROR;
	}


	/*check for "enforce consistent ACL"*/
	/*get the ACL Flags*/
	if (error = ACLGetFlags(hACL, &dwACLFlags))
	{
		OSMemFree (hACL);
		NSFDbClose (hDb);
		strcat (msg, " could not get ACL flags"); 
		LogMsg(msg);
		LogError (error);
		return (NOERROR);
	}

	/*check the flags - bail out if "consistent acl" is on*/
	if (dwACLFlags & ACL_UNIFORM_ACCESS)
	{
		strcat (msg, " no action - enforce consistent enabled");
		LogMsg (msg);
		OSMemFree (hACL);
		NSFDbClose (hDb);
		return (NOERROR);
	}

	/* add the new user (or group) to the ACL */
	/*make sure privilege bits are zeroed (ie. no roles set)*/
	for (i = 0; i < ACL_PRIVCOUNT; i++) 
		ACLClearPriv (PrivBits, i); 

	if (acl_info->remove_entry)
	{
		if (error = ACLDeleteEntry (hACL, acl_info->canonname))
		{
			OSMemFree (hACL);
			NSFDbClose (hDb);
			strcat (msg, " user name could not be removed");
			LogMsg (msg);
			LogError (error);
			return (NOERROR);
		}
	}
	else if (acl_info->flags & ACL_FLAG_ADMIN_SERVER)
	{
		/*change the admin server*/
		if (error = ACLSetAdminServer(hACL, acl_info->canonname))
		{
			strcat (msg, " administration server could not be changed");
			LogError (error);
			OSMemFree (hACL);
			NSFDbClose (hDb);
			return NOERROR;
		}
		/*check and change the admin servers control over reader/author fields*/
		if (acl_info->adminsrv_controls_fields && !(dwACLFlags & ACL_FLAG_ADMIN_READERAUTHOR))
			dwACLFlags |= ACL_FLAG_ADMIN_READERAUTHOR; /*turn feature on*/
		if (!acl_info->adminsrv_controls_fields && (dwACLFlags & ACL_FLAG_ADMIN_READERAUTHOR))
			dwACLFlags ^= ACL_FLAG_ADMIN_READERAUTHOR; /*flip feature off*/
		ACLSetFlags(hACL, dwACLFlags);
	}
	else if (strcmp(acl_info->canonname, ENTRYTYPE_DEFAULT) == 0)
	{
		/*alter the default user, use NULL as entry name*/
		if (error = ACLUpdateEntry (hACL, NULL, ACL_UPDATE_LEVEL | ACL_UPDATE_FLAGS, NULL,
			acl_info->level, NULL, acl_info->flags))
		{
			strcat (msg, " default user could not be changed");
			LogError (error);
			OSMemFree (hACL);
			NSFDbClose (hDb);
			return NOERROR;
		}
	}
	else
	{
		/*make sure user does not exist in ACL already*/
		ACLDeleteEntry (hACL, acl_info->canonname);
		/*then add them*/
		if (error = ACLAddEntry (hACL, /* handle to ACL */
							acl_info->canonname, /* Name to add */
							acl_info->level, /* Access level manager or reader or editor etc*/
							&PrivBits, /* Privilege bits */
							acl_info->flags)) /*modifier flags like no create and no delete*/
		{
			OSMemFree (hACL);
			NSFDbClose (hDb);
			strcat (msg, " user name could not be added");
			LogMsg (msg);
			LogError (error);
			return (NOERROR);
		}
	}

	/* save the ACL */
	if (error = NSFDbStoreACL (hDb,hACL,0L,0))
	{
		OSMemFree (hACL);
		NSFDbClose (hDb);
		strcat (msg, " ACL could not be saved"); 
		LogMsg(msg);
		LogError (error);
		return (NOERROR);
	}
	OSMemFree (hACL);
	NSFDbClose (hDb);

	strcat (msg, " was updated successfully");
	LogMsg(msg);

	return (NOERROR);
}

/************************************************************/
/* log a Notes error to the console */
/************************************************************/
void LogError (STATUS error)
{
	char error_msg[MSG_LENGTH];

	OSLoadString(NULLHANDLE, ERR(error), error_msg, sizeof(error_msg)-1);

	LogMsg (error_msg);

	return;
}

/************************************************************/
/* log a message to the console */
/************************************************************/
void LogMsg (char *usermsg)
{
	char msg[MSG_LENGTH];

	strcpy (msg, "ACLHelp: ");
	strcat (msg, usermsg);

#ifdef _DEBUG
	printf ("\n%s", msg);
#else
	AddInLogMessageText(msg, NOERROR);
#endif
	return;
}  
/************************************************************/
/*list the entries*/
void LNPUBLIC ListCallback (void *p, char far *Name, 
                                 WORD wAccessLevel, 
                                 ACL_PRIVILEGES far *pRoleBits, 
                                 WORD wAccessFlags)

{
	char canonname[MAXUSERNAME];
	WORD len;
	t_acl_info *acl_info = (t_acl_info *) p;
	if (wAccessLevel != acl_info->level) return;
	DNAbbreviate(0L, NULL, Name, canonname, MAXUSERNAME, &len);
	LogMsg(canonname);
}
STATUS ListEntries (HANDLE hACL, t_acl_info *acl_info)
{
	STATUS error;
	if(error = ACLEnumEntries (hACL, ListCallback, acl_info)) return error;
	return NOERROR;
}
void GetAccessLevelStr (WORD wAccessLevel, char *pAccessLevelName)

/*   GetAccessLevelStr - This function translates a given access level
 *                       number to the corresponding character string. 
 *                       The caller is responsible 
 *                       for providing adequate space for the string.
 *
 *   Inputs:
 *      wAccessLevel     - access level.
 *      
 *   Outputs:    
 *      pAccessLevelName - contains the returned access level name.
 */

{
   switch (wAccessLevel)
   {
      case ACL_LEVEL_NOACCESS:
         strcpy (pAccessLevelName, "No Access");
         break;
      case ACL_LEVEL_DEPOSITOR:
         strcpy (pAccessLevelName, "Depositor");
         break;
      case ACL_LEVEL_READER:
         strcpy (pAccessLevelName, "Reader");
         break;
      case ACL_LEVEL_AUTHOR:
         strcpy (pAccessLevelName, "Author");
         break;
      case ACL_LEVEL_EDITOR:
         strcpy (pAccessLevelName, "Editor");
         break;
      case ACL_LEVEL_DESIGNER:
         strcpy (pAccessLevelName, "Designer");
         break;
      case ACL_LEVEL_MANAGER:
         strcpy (pAccessLevelName, "Manager");
         break;
      default:
         strcpy (pAccessLevelName, "Unknown");

   } /* switch */
}
